@@ -32,6 +32,8 @@
 const ArtDRect eel_art_drect_empty = { 0.0, 0.0, 0.0, 0.0 };
 const ArtIRect eel_art_irect_empty = { 0, 0, 0, 0 };
 const ArtPoint eel_art_point_zero = { 0, 0 };
+const ArtPoint eel_art_point_max = { G_MAXDOUBLE, G_MAXDOUBLE };
+const ArtPoint eel_art_point_min = { G_MINDOUBLE, G_MINDOUBLE };
 const EelArtIPoint eel_art_ipoint_max = { G_MAXINT, G_MAXINT };
 const EelArtIPoint eel_art_ipoint_min = { G_MININT, G_MININT };
 const EelArtIPoint eel_art_ipoint_zero = { 0, 0 };
@@ -142,6 +144,35 @@ eel_art_irect_get_height (ArtIRect rectangle)
 	return rectangle.y1 - rectangle.y0;
 }
 
+
+/**
+ * eel_art_drect_get_width:
+ * 
+ * @rectangle: An ArtDrect.
+ *
+ * Returns: The width of the rectangle.
+ * 
+ */
+double
+eel_art_drect_get_width (ArtDRect rectangle)
+{
+	return rectangle.x1 - rectangle.x0;
+}
+
+/**
+ * eel_art_drect_get_height:
+ * 
+ * @rectangle: An ArtDRect.
+ *
+ * Returns: The height of the rectangle.
+ * 
+ */
+double
+eel_art_drect_get_height (ArtDRect rectangle)
+{
+	return rectangle.y1 - rectangle.y0;
+}
+
 /**
  * eel_art_irect_align:
  * 
@@ -233,6 +264,28 @@ eel_art_irect_assign_dimensions (int x,
 	return rectangle;
 }
 
+/**
+ * eel_art_irect_assign_end_points:
+ * 
+ * @top_left_point: Top left point.
+ * @bottom_right_point: Bottom right point.
+ *
+ * Returns: An ArtIRect that bounds the given end points.
+ */
+ArtIRect
+eel_art_irect_assign_end_points (EelArtIPoint top_left_point,
+				 EelArtIPoint bottom_right_point)
+{
+	ArtIRect rectangle;
+
+	rectangle.x0 = top_left_point.x;
+	rectangle.y0 = top_left_point.y;
+	rectangle.x1 = bottom_right_point.x;
+	rectangle.y1 = bottom_right_point.y;
+
+	return rectangle;
+}
+
 ArtIRect 
 eel_art_irect_offset_by (ArtIRect rectangle, int x, int y)
 {
@@ -344,6 +397,20 @@ eel_art_drect_inset (ArtDRect rectangle, double horizontal_inset, double vertica
 	return rectangle;
 }
 
+ArtDRect
+eel_art_drect_assign_end_points (ArtPoint top_left_point,
+				 ArtPoint bottom_right_point)
+{
+	ArtDRect rectangle;
+
+	rectangle.x0 = top_left_point.x;
+	rectangle.y0 = top_left_point.y;
+	rectangle.x1 = bottom_right_point.x;
+	rectangle.y1 = bottom_right_point.y;
+
+	return rectangle;
+}
+
 /**
  * eel_art_irect_intersect:
  * 
@@ -462,6 +529,43 @@ eel_art_ipoint_offset_by (EelArtIPoint point,
 	return eel_art_ipoint_assign (point.x + x, point.y + y);
 }
 
+gboolean
+eel_art_point_equal (ArtPoint point_a,
+		     ArtPoint point_b)
+{
+	return point_a.x == point_b.x
+		&& point_a.y == point_b.y;
+}
+
+ArtPoint
+eel_art_point_assign (double x,
+		      double y)
+{
+	ArtPoint point;
+
+	point.x = x;
+	point.y = y;
+
+	return point;
+}
+
+ArtPoint
+eel_art_point_clamp (ArtPoint point,
+		     ArtPoint min,
+		     ArtPoint max)
+{
+	return eel_art_point_assign (CLAMP (point.x, min.x, max.x),
+				     CLAMP (point.y, min.y, max.y));
+}
+
+ArtPoint
+eel_art_point_offset_by (ArtPoint point,
+			 double x,
+			 double y)
+{
+	return eel_art_point_assign (point.x + x, point.y + y);
+}
+
 #if !defined (EEL_OMIT_SELF_CHECK)
 
 static ArtIRect
