@@ -284,7 +284,7 @@ eel_drag_default_drop_action_for_icons (GdkDragContext *context,
 	/* Check for trash URI.  We do a find_directory for any Trash directory. */
 	if (eel_uri_is_trash (target_uri_string)) {
 		result = gnome_vfs_find_directory (NULL, GNOME_VFS_DIRECTORY_KIND_TRASH,
-						   &target_uri, FALSE, FALSE, 0777);
+						   &target_uri, TRUE, FALSE, 0777);
 		if (result != GNOME_VFS_OK) {
 			*default_action = 0;
 			*non_default_action = 0;
@@ -505,11 +505,14 @@ eel_drag_drop_action_ask (GdkDragAction actions)
 {
 	GtkWidget *menu;
 	int action;
+	char *saved_domain;
 
 	/* Create the menu and set the sensitivity of the items based on the
 	 * allowed actions.
 	 */
+	saved_domain = textdomain (GNOME_EXPLICIT_TRANSLATION_DOMAIN);
 	menu = gnome_popup_menu_new (menu_items);
+	textdomain (saved_domain);
 
 	gtk_widget_set_sensitive (menu_items[0].widget, (actions & GDK_ACTION_MOVE) != 0);
 	gtk_widget_set_sensitive (menu_items[1].widget, (actions & GDK_ACTION_COPY) != 0);
