@@ -33,9 +33,6 @@
 #include "eel-string-list.h"
 #include "eel-string.h"
 
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-
 #include <gdk/gdkprivate.h>
 #include <gdk/gdkx.h>
 
@@ -643,21 +640,37 @@ eel_gdk_font_get_largest_fitting (GdkFont *font,
 static char *
 eel_string_ellipsize_start (const char *string, GdkFont *font, int width)
 {
+	GdkWChar *wc_string;
+	gchar *mb_string;
+	char *result;
+
+	int wc_len;
 	int truncate_offset;
 	int resulting_width;
 
-	resulting_width = gdk_string_width (font, string);
+	wc_len = strlen (string) + 1;
+	wc_string = g_new0 (GdkWChar, wc_len);
+	wc_len = gdk_mbstowcs (wc_string, string, wc_len);
+
+	if (wc_len <= 0) {
+		g_free (wc_string);
+		return g_strdup("");
+	}
+
+	resulting_width = gdk_text_width_wc (font, wc_string, wc_len);
 	if (resulting_width <= width) {
 		/* String is already short enough. */
-		return g_strdup (string);
+		mb_string = gdk_wcstombs (wc_string);
+		g_free (wc_string);
+		return mb_string;
 	}
 	
 	/* Account for the width of the ellipsis. */
 	width -= gdk_string_width (font, ELLIPSIS);
-	
 
 	if (width < 0) {
 		/* No room even for a an ellipsis. */
+		g_free (wc_string);
 		return g_strdup ("");
 	}
 	
@@ -668,31 +681,53 @@ eel_string_ellipsize_start (const char *string, GdkFont *font, int width)
 	 * gdk_string_width ("foobar")
 	 */
         for (truncate_offset = 1; ; truncate_offset++) {
-        	if (string[truncate_offset] == '\0') {
+        	if (wc_string[truncate_offset] == L'\0') {
 			break;
         	}
 
-        	resulting_width -= gdk_char_width (font, string[truncate_offset - 1]);
+        	resulting_width -= gdk_char_width_wc (font, wc_string[truncate_offset - 1]);
 
         	if (resulting_width <= width) {
 			break;
         	}
         }
 
-	return g_strconcat (ELLIPSIS, string + truncate_offset, NULL);
+	mb_string = gdk_wcstombs (wc_string + truncate_offset);
+	result = g_strconcat (ELLIPSIS, mb_string, NULL);
+
+	g_free (mb_string);
+	g_free (wc_string);
+
+	return result;
 }
 
 static char *
 eel_string_ellipsize_end (const char *string, GdkFont *font, int width)
 {
-	int truncated_length;
+	GdkWChar *wc_string;
+	gchar *mb_string;
 	char *result;
+
+	int wc_len;
+
+	int truncated_length;
 	int resulting_width;
 
-	resulting_width = gdk_string_width (font, string);
+	wc_len = strlen (string) + 1;
+	wc_string = g_new0 (GdkWChar, wc_len);
+	wc_len = gdk_mbstowcs (wc_string, string, wc_len);
+
+	if (wc_len <= 0) {
+		g_free (wc_string);
+		return g_strdup("");
+	}
+
+	resulting_width = gdk_text_width_wc (font, wc_string, wc_len);
 	if (resulting_width <= width) {
 		/* String is already short enough. */
-		return g_strdup (string);
+		mb_string = gdk_wcstombs(wc_string);
+		g_free (wc_string);
+		return mb_string;
 	}
 	
 	/* Account for the width of the ellipsis. */
@@ -701,19 +736,24 @@ eel_string_ellipsize_end (const char *string, GdkFont *font, int width)
 
 	if (width < 0) {
 		/* No room even for a an ellipsis. */
+		g_free (wc_string);
 		return g_strdup ("");
 	}
 	
-        for (truncated_length = strlen (string) - 1; truncated_length > 0; truncated_length--) {
-        	resulting_width -= gdk_char_width (font, string[truncated_length]);
+        for (truncated_length = wc_len - 1; truncated_length > 0; truncated_length--) {
+        	resulting_width -= gdk_char_width_wc (font, wc_string[truncated_length]);
         	if (resulting_width <= width) {
 			break;
         	}
         }
-	
-	result = g_malloc (truncated_length + strlen (ELLIPSIS) + 1);
-	memcpy (result, string, truncated_length);
-	strcpy (result + truncated_length, ELLIPSIS);
+
+	wc_string [truncated_length] = L'\0';
+
+	mb_string = gdk_wcstombs (wc_string);
+	result = g_strconcat (mb_string, ELLIPSIS, NULL);
+
+	g_free (mb_string);
+	g_free (wc_string);
 
 	return result;
 }
@@ -721,29 +761,46 @@ eel_string_ellipsize_end (const char *string, GdkFont *font, int width)
 static char *
 eel_string_ellipsize_middle (const char *string, GdkFont *font, int width)
 {
+	GdkWChar *wc_string;
+	gchar *mb_string;
+	char *left, *right, *result;
+
+	int wc_len;
+
 	int original_length;
 	int starting_fragment_length;
 	int ending_fragment_offset;
 	int resulting_width;
-	char *result;
 
-	resulting_width = gdk_string_width (font, string);
+	wc_len = strlen (string) + 1;
+	wc_string = g_new0 (GdkWChar, wc_len);
+	wc_len = gdk_mbstowcs (wc_string, string, wc_len);
+
+	if( wc_len <= 0 ) {
+		g_free (wc_string);
+		return g_strdup("");
+	}
+
+	resulting_width = gdk_text_width_wc (font, wc_string, wc_len);
+
 	if (resulting_width <= width) {
 		/* String is already short enough. */
-		return g_strdup (string);
+		mb_string = gdk_wcstombs(wc_string);
+		g_free (wc_string);
+		return mb_string;
 	}
 	
 	/* Account for the width of the ellipsis. */
 	width -= gdk_string_width (font, ELLIPSIS);
-	
 
 	if (width < 0) {
 		/* No room even for a an ellipsis. */
+		g_free (wc_string);
 		return g_strdup ("");
 	}
 
 	/* Split the original string into two halves */
-	original_length = strlen (string);
+	original_length = wc_len;
 	
 	g_assert (original_length > 0);
 	
@@ -753,7 +810,7 @@ eel_string_ellipsize_middle (const char *string, GdkFont *font, int width)
 	/* Shave off a character at a time from the first and the second half
 	 * until we can fit
 	 */
-	resulting_width -= gdk_char_width (font, string[ending_fragment_offset - 1]);
+	resulting_width -= gdk_char_width_wc (font, wc_string[ending_fragment_offset - 1]);
 	
 	/* depending on whether the original string length is odd or even, start by
 	 * shaving off the characters from the starting or ending fragment
@@ -768,7 +825,7 @@ eel_string_ellipsize_middle (const char *string, GdkFont *font, int width)
 			if (starting_fragment_length > 0) {
 				starting_fragment_length--;
 			}
-			resulting_width -= gdk_char_width (font, string[starting_fragment_length]);
+			resulting_width -= gdk_char_width_wc (font, wc_string[starting_fragment_length]);
 	case 1:
 			if (resulting_width <= width) {
 				break;
@@ -777,16 +834,21 @@ eel_string_ellipsize_middle (const char *string, GdkFont *font, int width)
 			if (ending_fragment_offset < original_length) {
 				ending_fragment_offset++;
 			}
-			resulting_width -= gdk_char_width (font, string[ending_fragment_offset - 1]);
+			resulting_width -= gdk_char_width_wc (font, wc_string[ending_fragment_offset - 1]);
 	        }
 	}
-	
+
 	/* patch the two fragments together with an ellipsis */
-	result = g_malloc (starting_fragment_length + (original_length - ending_fragment_offset)
-		+ strlen (ELLIPSIS) + 1);
-	memcpy (result, string, starting_fragment_length);
-	strcpy (result + starting_fragment_length, ELLIPSIS);
-	strcpy (result + starting_fragment_length + strlen (ELLIPSIS), string + ending_fragment_offset);
+
+	right = gdk_wcstombs (wc_string+ending_fragment_offset);
+	wc_string [starting_fragment_length] = L'\0';
+	left = gdk_wcstombs (wc_string);
+
+	result = g_strconcat (left, ELLIPSIS, right, NULL);
+
+	g_free (wc_string);
+	g_free (left);
+	g_free (right);
 
 	return result;
 }
@@ -1164,6 +1226,7 @@ xlfd_string_get_nth_as_int (const char *xlfd_string,
 	}
 
 	if (eel_str_is_equal (nth, "*")) {
+		g_free (nth);
 		return XLFD_WILDCARD_VALUE;
 	}
 	
