@@ -41,7 +41,7 @@
 #include <math.h>
 #include <png.h>
 
-#define LOAD_BUFFER_SIZE 4096
+#define LOAD_BUFFER_SIZE 65536
 
 const ArtIRect eel_gdk_pixbuf_whole_pixbuf = { -1, -1, -1, -1 };
 
@@ -276,45 +276,62 @@ eel_cancel_gdk_pixbuf_load (EelPixbufLoadHandle *handle)
 }
 
 /* return the average value of each component */
-void 
-eel_gdk_pixbuf_average_value (GdkPixbuf *pixbuf, GdkColor *color)
+guint32
+eel_gdk_pixbuf_average_value_argb (GdkPixbuf *pixbuf)
 {
-	uint red_total, green_total, blue_total, count;
+	guint64 a_total, r_total, g_total, b_total;
 	int row, column;
+	gboolean has_alpha;
 	int width, height;
 	int row_stride;
-	guchar *pixsrc, *original_pixels;
-	
-	gboolean has_alpha;
-	
-	red_total = 0;
-	green_total = 0;
-	blue_total = 0;
-	count = 0;
+	const guchar *pixels, *p;
+	int r, g, b, a;
+	int dividend;
 
-	/* iterate through the pixbuf, counting up each component */
 	has_alpha = gdk_pixbuf_get_has_alpha (pixbuf);
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
 	row_stride = gdk_pixbuf_get_rowstride (pixbuf);
-	original_pixels = gdk_pixbuf_get_pixels (pixbuf);
-
+	pixels = gdk_pixbuf_get_pixels (pixbuf);
+	
+	/* iterate through the pixbuf, counting up each component */
+	a_total = 0;
+	r_total = 0;
+	g_total = 0;
+	b_total = 0;
 	for (row = 0; row < height; row++) {
-		pixsrc = original_pixels + (row * row_stride);
+		p = pixels + (row * row_stride);
 		for (column = 0; column < width; column++) {
-			red_total += *pixsrc++;
-			green_total += *pixsrc++;
-			blue_total += *pixsrc++;
-			count += 1;
-			if (has_alpha) {
-				pixsrc++;
-			}
+			r = *p++;
+			g = *p++;
+			b = *p++;
+			a = has_alpha ? *p++ : 0xFF;
+
+			a_total += a;
+			r_total += r * a;
+			g_total += g * a;
+			b_total += b * a;
 		}
 	}
 
-	color->red =   (red_total   * 256) / count;
-	color->green = (green_total * 256) / count;
-	color->blue =  (blue_total  * 256) / count;
+	a_total *= 0xFF;
+	dividend = height * width * 0xFF;
+	return ((a_total + dividend / 2) / dividend) << 24
+		| ((r_total + dividend / 2) / dividend) << 16
+		| ((g_total + dividend / 2) / dividend) << 8
+		| ((b_total + dividend / 2) / dividend);
+}
+
+void
+eel_gdk_pixbuf_average_value (GdkPixbuf *pixbuf, GdkColor *average_color)
+{
+	guint32 argb;
+
+	argb = eel_gdk_pixbuf_average_value_argb (pixbuf);
+	average_color->pixel = 0;
+	average_color->red = ((argb >> 16) & 0xFF) * 0x101;
+	average_color->green = ((argb >> 8) & 0xFF) * 0x101;
+	average_color->blue = (argb & 0xFF) * 0x101;
 }
 
 double
@@ -1497,6 +1514,75 @@ eel_gdk_pixbuf_intersect (const GdkPixbuf *pixbuf,
 
 #if !defined (EEL_OMIT_SELF_CHECK)
 
+static char *
+check_average_value (int width, int height, const char* fill)
+{
+	char c;
+	guint r, g, b, a;
+	gboolean alpha, gray;
+	int gray_tweak;
+	GdkPixbuf *pixbuf;
+	int x, y, rowstride, n_channels;
+	guchar *pixels;
+	guint32 average;
+	guchar v;
+
+	r = g = b = a = 0;
+	alpha = FALSE;
+	gray = FALSE;
+	gray_tweak = 0;
+	if (sscanf (fill, " %x,%x,%x,%x %c", &r, &g, &b, &a, &c) == 4) {
+		alpha = TRUE;
+	} else if (sscanf (fill, " %x,%x,%x %c", &r, &g, &b, &c) == 3) {
+	} else if (sscanf (fill, " gray%d %c", &gray_tweak, &c) == 1) {
+		gray = TRUE;
+	} else {
+		return g_strdup ("bad fill string format");
+	}
+
+	pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, alpha, 8, width, height);
+
+	pixels = gdk_pixbuf_get_pixels (pixbuf);
+	rowstride = gdk_pixbuf_get_rowstride (pixbuf);
+	n_channels = gdk_pixbuf_get_n_channels (pixbuf);
+
+	if (!gray) {
+		for (y = 0; y < height; y++) {
+			for (x = 0; x < width; x++) {
+				pixels [y * rowstride + x * n_channels + 0] = r;
+				pixels [y * rowstride + x * n_channels + 1] = g;
+				pixels [y * rowstride + x * n_channels + 2] = b;
+				if (alpha) {
+					pixels [y * rowstride + x * n_channels + 3] = a;
+				}
+			}
+		}
+	} else {
+		for (y = 0; y < height; y++) {
+			for (x = 0; x < width; x++) {
+				v = ((x + y) & 1) ? 0x80 : 0x7F;
+				if (((x + y) & 0xFF) == 0)
+					v += gray_tweak;
+				pixels [y * rowstride + x * n_channels + 0] = v;
+				pixels [y * rowstride + x * n_channels + 1] = v;
+				pixels [y * rowstride + x * n_channels + 2] = v;
+			}
+		}
+		pixels [0] += gray_tweak;
+		pixels [1] += gray_tweak;
+		pixels [2] += gray_tweak;
+	}
+
+	average = eel_gdk_pixbuf_average_value_argb (pixbuf);
+	gdk_pixbuf_unref (pixbuf);
+
+	return g_strdup_printf ("%02X,%02X,%02X,%02X",
+				(average >> 16) & 0xFF,
+				(average >> 8) & 0xFF,
+				average & 0xFF,
+				average >> 24);
+}
+
 void
 eel_self_check_gdk_pixbuf_extensions (void)
 {
@@ -1549,6 +1635,27 @@ eel_self_check_gdk_pixbuf_extensions (void)
 	EEL_CHECK_RECTANGLE_RESULT (eel_gdk_pixbuf_intersect (pixbuf, 0, 0, clip_area), 80, 0, 100, 100);
 
 	gdk_pixbuf_unref (pixbuf);
+
+	/* No checks for empty pixbufs because GdkPixbuf doesn't seem to allow them. */
+	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "00,00,00"), "00,00,00,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "00,00,00,00"), "00,00,00,00");
+	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "00,00,00,FF"), "00,00,00,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "01,01,01"), "01,01,01,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "FE,FE,FE"), "FE,FE,FE,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "FF,FF,FF"), "FF,FF,FF,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "FF,FF,FF,00"), "00,00,00,00");
+	EEL_CHECK_STRING_RESULT (check_average_value (1, 1, "11,22,33"), "11,22,33,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "00,00,00"), "00,00,00,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "00,00,00,00"), "00,00,00,00");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "00,00,00,FF"), "00,00,00,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "01,01,01"), "01,01,01,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "FE,FE,FE"), "FE,FE,FE,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "FF,FF,FF"), "FF,FF,FF,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "FF,FF,FF,00"), "00,00,00,00");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "11,22,33"), "11,22,33,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "gray -1"), "7F,7F,7F,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "gray 0"), "80,80,80,FF");
+	EEL_CHECK_STRING_RESULT (check_average_value (1000, 1000, "gray 1"), "80,80,80,FF");
 }
 
 #endif /* !EEL_OMIT_SELF_CHECK */
