@@ -28,6 +28,7 @@
 #include "eel-art-extensions.h"
 #include "eel-art-gtk-extensions.h"
 #include "eel-debug-drawing.h"
+#include "eel-gtk-container.h"
 #include "eel-gtk-extensions.h"
 #include "eel-gtk-macros.h"
 #include "eel-image.h"
@@ -91,6 +92,8 @@ static void          eel_labeled_image_size_request       (GtkWidget
 							   GtkRequisition        *requisition);
 static int           eel_labeled_image_expose_event       (GtkWidget             *widget,
 							   GdkEventExpose        *event);
+static void          eel_labeled_image_draw               (GtkWidget             *widget,
+							   GdkRectangle          *area);
 static void          eel_labeled_image_size_allocate      (GtkWidget             *widget,
 							   GtkAllocation         *allocation);
 static void          eel_labeled_image_map                (GtkWidget             *widget);
@@ -136,6 +139,7 @@ eel_labeled_image_initialize_class (EelLabeledImageClass *labeled_image_class)
  	widget_class->size_request = eel_labeled_image_size_request;
 	widget_class->size_allocate = eel_labeled_image_size_allocate;
  	widget_class->expose_event = eel_labeled_image_expose_event;
+ 	widget_class->draw = eel_labeled_image_draw;
 	widget_class->map = eel_labeled_image_map;
 	widget_class->unmap = eel_labeled_image_unmap;
 
@@ -399,28 +403,14 @@ eel_labeled_image_size_allocate (GtkWidget *widget,
 	widget->allocation = *allocation;
 	
  	label_bounds = eel_labeled_image_get_label_bounds (labeled_image);
-	if (!art_irect_empty (&label_bounds)) {
-		GtkAllocation label_allocation;
-
-		label_allocation.x = label_bounds.x0;
-		label_allocation.y = label_bounds.y0;
-		label_allocation.width = eel_art_irect_get_width (label_bounds);
-		label_allocation.height = eel_art_irect_get_height (label_bounds);
-
-		gtk_widget_size_allocate (labeled_image->details->label, &label_allocation);
-	}
+	eel_gtk_container_child_size_allocate (GTK_CONTAINER (widget),
+					       labeled_image->details->label,
+					       label_bounds);
 
  	image_bounds = eel_labeled_image_get_image_bounds (labeled_image);
-	if (!art_irect_empty (&image_bounds)) {
-		GtkAllocation image_allocation;
-		
-		image_allocation.x = image_bounds.x0;
-		image_allocation.y = image_bounds.y0;
-		image_allocation.width = eel_art_irect_get_width (image_bounds);
-		image_allocation.height = eel_art_irect_get_height (image_bounds);
-
-		gtk_widget_size_allocate (labeled_image->details->image, &image_allocation);
-	}
+	eel_gtk_container_child_size_allocate (GTK_CONTAINER (widget),
+					       labeled_image->details->image,
+					       image_bounds);
 }
 
 static int
@@ -434,34 +424,46 @@ eel_labeled_image_expose_event (GtkWidget *widget,
 	g_return_val_if_fail (event != NULL, TRUE);
 
   	labeled_image = EEL_LABELED_IMAGE (widget);
-
+	
 	if (labeled_image_show_label (labeled_image)) {
-		GdkEventExpose label_event;
-		
-		label_event = *event;
-
-		if (GTK_WIDGET_DRAWABLE (labeled_image->details->label) &&
-		    GTK_WIDGET_NO_WINDOW (labeled_image->details->label) &&
-		    gtk_widget_intersect (labeled_image->details->label, &event->area, &label_event.area)) {
-			gtk_widget_event (labeled_image->details->label, (GdkEvent *) &label_event);
-		}
+		eel_gtk_container_child_expose_event (GTK_CONTAINER (widget),
+						      labeled_image->details->label,
+						      event);
 	}
 	
 	if (labeled_image_show_image (labeled_image)) {
-		GdkEventExpose image_event;
-
-		image_event = *event;
-
-		if (GTK_WIDGET_DRAWABLE (labeled_image->details->image) &&
-		    GTK_WIDGET_NO_WINDOW (labeled_image->details->image) &&
-		    gtk_widget_intersect (labeled_image->details->image, &event->area, &image_event.area)) {
-			gtk_widget_event (labeled_image->details->image, (GdkEvent *) &image_event);
-		}
+		eel_gtk_container_child_expose_event (GTK_CONTAINER (widget),
+						      labeled_image->details->image,
+						      event);
 	}
 
 	return FALSE;
 }
 
+static void
+eel_labeled_image_draw (GtkWidget *widget,
+			GdkRectangle *area)
+{
+	EelLabeledImage *labeled_image;
+	
+	g_return_if_fail (EEL_IS_LABELED_IMAGE (widget));
+	g_return_if_fail (area != NULL);
+	
+  	labeled_image = EEL_LABELED_IMAGE (widget);
+	
+	if (labeled_image_show_label (labeled_image)) {
+		eel_gtk_container_child_draw (GTK_CONTAINER (widget),
+					      labeled_image->details->label,
+					      area);
+	}
+	
+	if (labeled_image_show_image (labeled_image)) {
+		eel_gtk_container_child_draw (GTK_CONTAINER (widget),
+					      labeled_image->details->image,
+					      area);
+	}
+}
+
 static void
 eel_labeled_image_map (GtkWidget *widget)
 {
@@ -474,17 +476,11 @@ eel_labeled_image_map (GtkWidget *widget)
  	GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
 	
 	if (labeled_image_show_label (labeled_image)) {
-		if (GTK_WIDGET_VISIBLE (labeled_image->details->label) &&
-		    !GTK_WIDGET_MAPPED (labeled_image->details->label)) {
-			gtk_widget_map (labeled_image->details->label);
-		}
+		eel_gtk_container_child_map (GTK_CONTAINER (widget), labeled_image->details->label);
 	}
 
 	if (labeled_image_show_image (labeled_image)) {
-		if (GTK_WIDGET_VISIBLE (labeled_image->details->image) &&
-		    !GTK_WIDGET_MAPPED (labeled_image->details->image)) {
-			gtk_widget_map (labeled_image->details->image);
-		}
+		eel_gtk_container_child_map (GTK_CONTAINER (widget), labeled_image->details->image);
 	}
 }
 
@@ -499,19 +495,8 @@ eel_labeled_image_unmap (GtkWidget *widget)
 
 	GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
 	
-	if (labeled_image->details->label != NULL) {
-		if (GTK_WIDGET_VISIBLE (labeled_image->details->label) &&
-		    GTK_WIDGET_MAPPED (labeled_image->details->label)) {
-			gtk_widget_unmap (labeled_image->details->label);
-		}
-	}
-
-	if (labeled_image->details->image != NULL) {
-		if (GTK_WIDGET_VISIBLE (labeled_image->details->image) &&
-		    GTK_WIDGET_MAPPED (labeled_image->details->image)) {
-			gtk_widget_unmap (labeled_image->details->image);
-		}
-	}
+	eel_gtk_container_child_unmap (GTK_CONTAINER (widget), labeled_image->details->label);
+	eel_gtk_container_child_unmap (GTK_CONTAINER (widget), labeled_image->details->image);
 }
 
 /* GtkContainerClass methods */
@@ -519,23 +504,10 @@ static void
 eel_labeled_image_add (GtkContainer *container,
 		       GtkWidget *child)
 {
-	g_return_if_fail (container != NULL);
 	g_return_if_fail (EEL_IS_LABELED_IMAGE (container));
 	g_return_if_fail (EEL_IS_LABEL (child) || EEL_IS_IMAGE (child));
 
-	gtk_widget_set_parent (child, GTK_WIDGET (container));
-
-	if (GTK_WIDGET_REALIZED (container)) {
-		gtk_widget_realize (child);
-	}
-	
-	if (GTK_WIDGET_VISIBLE (container) && GTK_WIDGET_VISIBLE (child)) {
-		if (GTK_WIDGET_MAPPED (container)) {
-			gtk_widget_map (child);
-		}
-		
-		gtk_widget_queue_resize (child);
-	}
+	eel_gtk_container_child_add (container, child);
 }
 
 static void
@@ -543,7 +515,6 @@ eel_labeled_image_remove (GtkContainer *container,
 			  GtkWidget *child)
 {
 	EelLabeledImage *labeled_image;
-	gboolean child_was_visible;
 	
 	g_return_if_fail (EEL_IS_LABELED_IMAGE (container));
 	g_return_if_fail (EEL_IS_LABEL (child) || EEL_IS_IMAGE (child));
@@ -552,13 +523,7 @@ eel_labeled_image_remove (GtkContainer *container,
 
 	g_return_if_fail (child == labeled_image->details->image || child == labeled_image->details->label);
 
-	child_was_visible = GTK_WIDGET_VISIBLE (child);
-
-	gtk_widget_unparent (child);
-
-	if (child_was_visible) {
-		gtk_widget_queue_resize (GTK_WIDGET (container));
-	}
+	eel_gtk_container_child_remove (container, child);
 
 	if (labeled_image->details->image == child) {
 		labeled_image->details->image = NULL;
