@@ -68,6 +68,7 @@ struct EelPreferencesItemDetails
 typedef enum
 {
 	CUSTOM_UPDATE_DISPLAYED_VALUE,
+	CUSTOM_DESCRIPTION_CHANGED,
 	LAST_SIGNAL
 } EelStringListSignals;
 
@@ -88,12 +89,12 @@ static void preferences_item_create_enumeration_radio                 (EelPrefer
 static void preferences_item_create_enumeration_list                  (EelPreferencesItem      *item,
 								       gboolean                 horizontal);
 static void preferences_item_create_font                              (EelPreferencesItem      *item);
-static void preferences_item_create_font                              (EelPreferencesItem      *item);
 static void preferences_item_create_padding                           (EelPreferencesItem      *item);
 static void preferences_item_create_smooth_font                       (EelPreferencesItem      *item);
 static void preferences_item_update_displayed_value                   (EelPreferencesItem      *preferences_item);
 static void preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem      *preferences_item);
 static void preferences_item_update_text_settings_at_idle             (EelPreferencesItem      *preferences_item);
+static void preferences_item_update_description                       (EelPreferencesItem      *preferences_item);
 
 /* User triggered item changed callbacks */
 static void enumeration_radio_changed_callback                        (EelRadioButtonGroup     *radio_button_group,
@@ -137,7 +138,16 @@ eel_preferences_item_initialize_class (EelPreferencesItemClass *preferences_item
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 
 				0);
-
+	preferences_item_signals[CUSTOM_DESCRIPTION_CHANGED] = 
+		gtk_signal_new ("custom_description_changed",
+				GTK_RUN_LAST,
+				object_class->type,
+				0,
+				gtk_marshal_NONE__POINTER_POINTER,
+				GTK_TYPE_NONE, 
+				2,
+				GTK_TYPE_OBJECT,
+				GTK_TYPE_STRING);
 	gtk_object_class_add_signals (object_class, preferences_item_signals, LAST_SIGNAL);
 }
 
@@ -325,7 +335,6 @@ preferences_item_update_custom (EelPreferencesItem *item)
 
 	gtk_signal_emit (GTK_OBJECT (item),
 			 preferences_item_signals[CUSTOM_UPDATE_DISPLAYED_VALUE]);
-	
 }
 
 /* This callback is called whenever the preference value changes, so that we can
@@ -364,6 +373,8 @@ preferences_item_set_main_child (EelPreferencesItem *item,
 	gtk_widget_show (child);
 
 	item->details->main_child = child;
+
+	preferences_item_update_description (item);
 }
 
 static void
@@ -434,7 +445,6 @@ preferences_item_create_enumeration_list (EelPreferencesItem *item,
  	guint i;
  	guint j;
 	char *enumeration_id;
-	char *description;
 	char *enum_description;
 	EelStringList *default_value;
 	guint num_pickers;
@@ -446,9 +456,6 @@ preferences_item_create_enumeration_list (EelPreferencesItem *item,
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (eel_strlen (description) > 0);
-
 	enumeration_id = eel_preferences_get_enumeration_id (item->details->preference_name);
 	g_return_if_fail (eel_strlen (enumeration_id) > 0);
 	g_return_if_fail (eel_enumeration_id_get_length (enumeration_id) > 0);
@@ -462,7 +469,7 @@ preferences_item_create_enumeration_list (EelPreferencesItem *item,
 	child = gtk_vbox_new (FALSE, 4);
 	picker_box = horizontal ? gtk_hbox_new (FALSE, 4): gtk_vbox_new (FALSE, 4);
 
-	title = gtk_label_new (description);
+	title = gtk_label_new ("");
 	gtk_misc_set_alignment (GTK_MISC (title), 0.0, 0.5);
 	gtk_label_set_justify (GTK_LABEL (title), GTK_JUSTIFY_LEFT);
 
@@ -531,18 +538,13 @@ preferences_item_update_boolean (EelPreferencesItem *item)
 static void
 preferences_item_create_boolean (EelPreferencesItem *item)
 {
-	char *description;
 	GtkWidget *child;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (eel_strlen (description) > 0);
-
-	child = gtk_check_button_new_with_label (description);
+	child = gtk_check_button_new_with_label ("");
 	gtk_label_set_justify (GTK_LABEL (GTK_BIN (child)->child), GTK_JUSTIFY_LEFT);
-	g_free (description);
 	
 	preferences_item_add_connection_child (item,
 					       child,
@@ -563,22 +565,23 @@ preferences_item_update_editable_string (EelPreferencesItem *item)
 	current_value = eel_preferences_get (item->details->preference_name);
 
 	g_assert (current_value != NULL);
-	eel_text_caption_set_text (EEL_TEXT_CAPTION (item->details->main_child), current_value);
+	if (strcmp (eel_text_caption_get_text (EEL_TEXT_CAPTION (item->details->main_child)),
+		    current_value) != 0) {
+		eel_text_caption_set_text (EEL_TEXT_CAPTION (item->details->main_child), 
+					   current_value);
+	}
+
 	g_free (current_value);
 }
 
 static void
 preferences_item_create_editable_string (EelPreferencesItem *item)
 {
-	char *description;
 	GtkWidget *child;
 	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_assert (description != NULL);
-	
 	child = eel_text_caption_new ();
 
  	/* FIXME:
@@ -589,9 +592,7 @@ preferences_item_create_editable_string (EelPreferencesItem *item)
  		eel_text_caption_set_expand_tilde (EEL_TEXT_CAPTION (child), TRUE);
  	}
 	
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-
-	g_free (description);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 	
 	preferences_item_add_connection_child (item,
 					       child,
@@ -612,28 +613,27 @@ preferences_item_update_editable_integer (EelPreferencesItem *item)
 	current_value = g_strdup_printf ("%d", eel_preferences_get_integer (item->details->preference_name));
 
 	g_assert (current_value != NULL);
-	eel_text_caption_set_text (EEL_TEXT_CAPTION (item->details->main_child), current_value);
+
+	if (strcmp (eel_text_caption_get_text (EEL_TEXT_CAPTION (item->details->main_child)),
+		    current_value) != 0) {
+		eel_text_caption_set_text (EEL_TEXT_CAPTION (item->details->main_child), 
+					   current_value);
+	}
+
 	g_free (current_value);
 }
 
 static void
 preferences_item_create_editable_integer (EelPreferencesItem *item)
 {
-	char *description;
 	GtkWidget *child;
 	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	
-	g_assert (description != NULL);
-	
 	child = eel_text_caption_new ();
 
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-
-	g_free (description);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 
 	preferences_item_add_connection_child (item,
 					       child,
@@ -686,18 +686,14 @@ preferences_item_create_enumeration_menu (EelPreferencesItem *item)
 {
  	guint i;
 	char *enumeration_id;
-	char *description;
 	GtkWidget *child;
+	char *enum_description;
 	
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 	
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (description != NULL);
-	
 	child = eel_string_picker_new ();
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-	g_free (description);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 
 	enumeration_id = eel_preferences_get_enumeration_id (item->details->preference_name);
 	g_return_if_fail (eel_strlen (enumeration_id) > 0);
@@ -705,16 +701,16 @@ preferences_item_create_enumeration_menu (EelPreferencesItem *item)
 	
 	/* Populate the string picker */
 	for (i = 0; i < eel_enumeration_id_get_length (enumeration_id); i++) {
-		description = eel_enumeration_id_get_nth_description_translated (enumeration_id, i);
-		g_assert (description != NULL);
+		enum_description = eel_enumeration_id_get_nth_description_translated (enumeration_id, i);
+		g_assert (enum_description != NULL);
 		
-		if (description[0] == '-') {
+		if (enum_description[0] == '-') {
 			eel_string_picker_insert_separator (EEL_STRING_PICKER (child));
 		} else {
-			eel_string_picker_insert_string (EEL_STRING_PICKER (child), description);
+			eel_string_picker_insert_string (EEL_STRING_PICKER (child), enum_description);
 		}
 		
-		g_free (description);
+		g_free (enum_description);
 	}
  	g_free (enumeration_id);
 
@@ -753,21 +749,15 @@ preferences_item_update_font (EelPreferencesItem *item)
 static void
 preferences_item_create_font (EelPreferencesItem *item)
 {
-	char *description;
 	EelStringList *font_list;
 	GtkWidget *child;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (description != NULL);
-	
 	child = eel_string_picker_new ();
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 	
-	g_free (description);
-
 	/* FIXME bugzilla.eazel.com 1274: Need to query system for available fonts */
 	font_list = eel_string_list_new (TRUE);
 
@@ -847,18 +837,13 @@ smooth_font_changed_callback (EelFontPicker *font_picker,
 static void
 preferences_item_create_smooth_font (EelPreferencesItem *item)
 {
-	char *description = NULL;
 	GtkWidget *child;
 
 	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
 
-	description = eel_preferences_get_description (item->details->preference_name);
-	g_return_if_fail (description != NULL);
-	
 	child = eel_font_picker_new ();
-	eel_caption_set_title_label (EEL_CAPTION (child), description);
-	g_free (description);
+	eel_caption_set_title_label (EEL_CAPTION (child), "");
 
 	preferences_item_add_connection_child (item,
 					       child,
@@ -1160,11 +1145,11 @@ enumeration_list_changed_callback (EelStringPicker *string_picker,
 }
 
 char *
-eel_preferences_item_get_name (const EelPreferencesItem *preferences_item)
+eel_preferences_item_get_name (const EelPreferencesItem *item)
 {
-	g_return_val_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item), NULL);
+	g_return_val_if_fail (EEL_IS_PREFERENCES_ITEM (item), NULL);
 
-	return g_strdup (preferences_item->details->preference_name);
+	return g_strdup (item->details->preference_name);
 }
 
 static void
@@ -1248,14 +1233,14 @@ preferences_item_update_displayed_value (EelPreferencesItem *item)
 }
 
 static gboolean
-update_text_settings_at_idle (EelPreferencesItem *preferences_item)
+update_text_settings_at_idle (EelPreferencesItem *item)
 {
 	char *text;
 
-	text = eel_text_caption_get_text (EEL_TEXT_CAPTION (preferences_item->details->main_child));
+	text = eel_text_caption_get_text (EEL_TEXT_CAPTION (item->details->main_child));
 
 	if (text != NULL) {
-		eel_preferences_set (preferences_item->details->preference_name, text);
+		eel_preferences_set (item->details->preference_name, text);
 		g_free (text);
 	}
 	
@@ -1265,29 +1250,29 @@ update_text_settings_at_idle (EelPreferencesItem *preferences_item)
 }
 
 static void
-preferences_item_update_text_settings_at_idle (EelPreferencesItem *preferences_item)
+preferences_item_update_text_settings_at_idle (EelPreferencesItem *item)
 {
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item));
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 
 	if (text_idle_handler == FALSE) {
-		gtk_idle_add ((GtkFunction) update_text_settings_at_idle, preferences_item);
+		gtk_idle_add ((GtkFunction) update_text_settings_at_idle, item);
 		text_idle_handler = TRUE;
 	}
 }
 
 static gboolean
-update_integer_settings_at_idle (EelPreferencesItem *preferences_item)
+update_integer_settings_at_idle (EelPreferencesItem *item)
 {
 	int value = 0;
 	char *text;
 
-	text = eel_text_caption_get_text (EEL_TEXT_CAPTION (preferences_item->details->main_child));
+	text = eel_text_caption_get_text (EEL_TEXT_CAPTION (item->details->main_child));
 
 	if (text != NULL) {
 		eel_eat_str_to_int (text, &value);
 	}
 	
-	eel_preferences_set_integer (preferences_item->details->preference_name, value);
+	eel_preferences_set_integer (item->details->preference_name, value);
 
 	integer_idle_handler = FALSE;
 
@@ -1295,64 +1280,78 @@ update_integer_settings_at_idle (EelPreferencesItem *preferences_item)
 }
 
 static void
-preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem *preferences_item)
+preferences_item_update_editable_integer_settings_at_idle (EelPreferencesItem *item)
 {
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item));
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 
 	if (integer_idle_handler == FALSE) {
-		gtk_idle_add ((GtkFunction) update_integer_settings_at_idle, preferences_item);
+		gtk_idle_add ((GtkFunction) update_integer_settings_at_idle, item);
 		integer_idle_handler = TRUE;
 	}
 }
 
+static void
+preferences_item_update_description (EelPreferencesItem *item)
+{
+	char *description;
+
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
+	g_return_if_fail (eel_strlen (item->details->preference_name) > 0);
+	
+	description = eel_preferences_get_description (item->details->preference_name);
+	g_return_if_fail (description != NULL);
+	eel_preferences_item_set_description (item, description);
+	g_free (description);
+}
+
 void
-eel_preferences_item_set_control_preference (EelPreferencesItem *preferences_item,
+eel_preferences_item_set_control_preference (EelPreferencesItem *item,
 					     const char *control_preference_name)
 {
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item));
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 
-	if (eel_str_is_equal (preferences_item->details->control_preference_name,
+	if (eel_str_is_equal (item->details->control_preference_name,
 			      control_preference_name)) {
 		return;
 	}
 
-	g_free (preferences_item->details->control_preference_name);
-	preferences_item->details->control_preference_name = g_strdup (control_preference_name);
+	g_free (item->details->control_preference_name);
+	item->details->control_preference_name = g_strdup (control_preference_name);
 }
 
 void
-eel_preferences_item_set_control_action (EelPreferencesItem *preferences_item,
+eel_preferences_item_set_control_action (EelPreferencesItem *item,
 					 EelPreferencesItemControlAction control_action)
 {
-	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item));
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
 	g_return_if_fail (control_action >= EEL_PREFERENCE_ITEM_SHOW);
 	g_return_if_fail (control_action <= EEL_PREFERENCE_ITEM_HIDE);
 
-	if (preferences_item->details->control_action == control_action) {
+	if (item->details->control_action == control_action) {
 		return;
 	}
 
-	preferences_item->details->control_action = control_action;
+	item->details->control_action = control_action;
 }
 
 static gboolean
-preferences_item_get_control_showing (const EelPreferencesItem *preferences_item)
+preferences_item_get_control_showing (const EelPreferencesItem *item)
 {
 	gboolean value;
 
-	g_return_val_if_fail (EEL_IS_PREFERENCES_ITEM (preferences_item), FALSE);
+	g_return_val_if_fail (EEL_IS_PREFERENCES_ITEM (item), FALSE);
 
-	if (preferences_item->details->control_preference_name == NULL) {
+	if (item->details->control_preference_name == NULL) {
 		return TRUE;
 	}
 
-	value = eel_preferences_get_boolean (preferences_item->details->control_preference_name);
+	value = eel_preferences_get_boolean (item->details->control_preference_name);
 
-	if (preferences_item->details->control_action == EEL_PREFERENCE_ITEM_SHOW) {
+	if (item->details->control_action == EEL_PREFERENCE_ITEM_SHOW) {
 		return value;
 	}
 
-	if (preferences_item->details->control_action == EEL_PREFERENCE_ITEM_HIDE) {
+	if (item->details->control_action == EEL_PREFERENCE_ITEM_HIDE) {
 		return !value;
 	}
 
@@ -1447,3 +1446,56 @@ eel_preferences_item_enumeration_list_set_unique_exceptions (EelPreferencesItem
 
 	preferences_item_update_enumeration_list_uniqueness (item);
 }
+
+void
+eel_preferences_item_set_description (EelPreferencesItem *item,
+				      const char *description)
+{
+	GList *node;
+
+	g_return_if_fail (EEL_IS_PREFERENCES_ITEM (item));
+	g_return_if_fail (description != NULL);
+
+	switch (item->details->item_type) {
+	case EEL_PREFERENCE_ITEM_BOOLEAN:
+		g_assert (GTK_IS_CHECK_BUTTON (item->details->main_child));
+		g_assert (GTK_IS_LABEL (GTK_BIN (item->details->main_child)->child));
+		gtk_label_set_text (GTK_LABEL (GTK_BIN (item->details->main_child)->child),
+				    description);
+		break;
+	case EEL_PREFERENCE_ITEM_ENUMERATION_VERTICAL_RADIO:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_HORIZONTAL_RADIO:
+		/* These 2 dont have a visible description.  */
+		break;
+	case EEL_PREFERENCE_ITEM_ENUMERATION_LIST_VERTICAL:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_LIST_HORIZONTAL:
+		g_assert (GTK_IS_BOX (item->details->main_child));
+		node = g_list_first (GTK_BOX (item->details->main_child)->children);
+		g_assert (node != NULL);
+		g_assert (node->data != NULL);
+		g_assert (GTK_IS_LABEL (((GtkBoxChild *) node->data)->widget));
+		gtk_label_set_text (GTK_LABEL (((GtkBoxChild *) node->data)->widget),
+				    description);
+		break;
+	case EEL_PREFERENCE_ITEM_FONT:
+	case EEL_PREFERENCE_ITEM_SMOOTH_FONT:
+	case EEL_PREFERENCE_ITEM_EDITABLE_STRING:
+	case EEL_PREFERENCE_ITEM_EDITABLE_INTEGER:
+	case EEL_PREFERENCE_ITEM_ENUMERATION_MENU:
+		g_assert (EEL_IS_CAPTION (item->details->main_child));
+		eel_caption_set_title_label (EEL_CAPTION (item->details->main_child),
+					     description);
+		break;	
+	case EEL_PREFERENCE_ITEM_PADDING:
+		/* Padding doesnt have a description */
+		break;
+	case EEL_PREFERENCE_ITEM_CUSTOM:
+		gtk_signal_emit (GTK_OBJECT (item),
+				 preferences_item_signals[CUSTOM_DESCRIPTION_CHANGED],
+				 item->details->main_child,
+				 description);
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+}
